#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  bmp085.py
#  
#  Copyright 2013 Stefan Mavrodiev <support@olimex.com>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.

from optparse import *
from time import sleep
import smbus
import sys


#define class for printing colors
class colors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    BOLD = '\033[1m'
    ENDC = '\033[0m'
    

#define some global variables
BUS = None
UT = None
UP = None

AC1 = None
AC2 = None
AC3 = None
AC4 = None
AC5 = None
AC6 = None
B1 = None
B2 = None
MB = None
MC = None
MD = None

usage = "usage: %prog [options] arg"
parser = OptionParser()
parser.add_option("-i", "--i2c", 
                    dest="i2c", 
                    action="store",
                    default=2,
                    type="int",
                    help="Select i2c-bus: 0, 1, 2 etc")
parser.add_option("-o", "--oversampling",
                    dest="oss",
                    action="store",
                    default=1,
                    type="int",
                    help="Select oversampling rate")
parser.add_option("-a", "--address", 
                    dest="address",
                    action="store",
                    default=0x77,
                    type="int", 
                    help="Set i2c address")
parser.add_option("-v", "--verbose",
                    dest="verbose",
                    action="store_true",
                    default=False,
                    help="Print some extra messages for debugging")

#Add some debug options                    
group = OptionGroup(parser, "Debug options")
group.add_option("-d", "--debug",
                    action="store_true",
                    help="Print debug information")
group.add_option("-e",
                    action="store_true",
                    default=False,
                    help="Print everything")
parser.add_option_group(group)

(options, args) = parser.parse_args()

#Handle errors in options
if options.debug and options.e:
    parser.error("options -e and -d are mutually exclusive")
    
if options.oss < 0 or options.oss > 3:
    parser.error("oversampling must be 0, 1, 2 or 3")
    
if options.address < 0 or options.address > 0x77:
    parser.error("invalid device address")
    

def toSigned(val):
    if val & 0x8000:
        return (val - (0xFFFF + 1))
    else:
        return val
        

def init():
    #Initialize MOD-BMP085
    if options.debug or options.e:
        print (colors.BOLD + "Initializating MOD-BMP085:" + colors.ENDC),
        
        
    try:
        #define variable for i2c access
        if options.e:
            print
            print("Opening i2c-bus: "),
        global BUS
        BUS = smbus.SMBus(options.i2c)
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #read calibration data
        if options.e:
            print("Read calibration data: "),
        calibration_data = BUS.read_i2c_block_data(options.address, 0xAA, 22)
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #convert calibration data
        if options.e:
            print("Parsing calibration data: "),
        global AC1
        AC1 = toSigned(calibration_data[0] << 8 | calibration_data[1])
        global AC2
        AC2 = toSigned(calibration_data[2] << 8 | calibration_data[3])
        global AC3
        AC3 = toSigned(calibration_data[4] << 8 | calibration_data[5])
        global AC4
        AC4 = calibration_data[6] << 8 | calibration_data[7]
        global AC5
        AC5 = calibration_data[8] << 8 | calibration_data[9]
        global AC6
        AC6 = calibration_data[10] << 8 | calibration_data[11]
        global B1
        B1 = calibration_data[12] << 8 | calibration_data[13]
        global B2
        B2 = toSigned(calibration_data[14] << 8 | calibration_data[15])
        global MB
        MB = toSigned(calibration_data[16] << 8 | calibration_data[17])
        global MC
        MC = toSigned(calibration_data[18] << 8 | calibration_data[19])
        global MD
        MD = toSigned(calibration_data[20] << 8 | calibration_data[21])
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #print calibration data
        if options.e:
            print
            print("Calibration data:")
            print '='*40
            print("AC1 = " + str(calibration_data[0]) + " << 8 | " + str(calibration_data[1]) + " = " + str(AC1))
            print("AC2 = " + str(calibration_data[2]) + " << 8 | " + str(calibration_data[3]) + " = " + str(AC2))
            print("AC3 = " + str(calibration_data[4]) + " << 8 | " + str(calibration_data[5]) + " = " + str(AC3))
            print("AC4 = " + str(calibration_data[6]) + " << 8 | " + str(calibration_data[7]) + " = " + str(AC4))
            print("AC5 = " + str(calibration_data[8]) + " << 8 | " + str(calibration_data[9]) + " = " + str(AC5))
            print("AC6 = " + str(calibration_data[10]) + " << 8 | " + str(calibration_data[11]) + " = " + str(AC6))
            print("B1 = " + str(calibration_data[12]) + " << 8 | " + str(calibration_data[13]) + " = " + str(B1))
            print("B2 = " + str(calibration_data[14]) + " << 8 | " + str(calibration_data[15]) + " = " + str(B2))
            print("MB = " + str(calibration_data[16]) + " << 8 | " + str(calibration_data[17]) + " = " + str(MB))
            print("MC = " + str(calibration_data[18]) + " << 8 | " + str(calibration_data[19]) + " = " + str(MC))
            print("MD = " + str(calibration_data[20]) + " << 8 | " + str(calibration_data[21]) + " = " + str(MD))
            print '='*40
            print
            
        if options.debug:
            print colors.OKGREEN + "OK" + colors.ENDC
            
    except Exception, ex:
        print colors.FAIL + "FAIL" + colors.ENDC
        print '='*40
        print str(ex)
        sys.exit(1)  
            
        
    return
    
def read_data():
    #read uncompensated data
    if options.debug or options.e:
        print (colors.BOLD + "Reading uncompensated temperature value: " + colors.ENDC),
    try:
        #send command
        if options.e:
            print
            print("Sending command: "),
        BUS.write_i2c_block_data(options.address, 0xF4, [0x2E])
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #wait
        if options.e:
            print("Sleep: "),
        sleep(0.05)
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #read data
        if options.e:
            print("Reading data: "),
        raw_temp = BUS.read_i2c_block_data(options.address, 0xF6, 2)
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #parse raw temperature
        if options.e:
            print("Converting raw data: "),
        global UT
        UT = raw_temp[0] << 8 | raw_temp[1]
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #print temperature
        if options.e:
            print
            print("Temperature data:")
            print '='*40
            print("UT = " + str(raw_temp[0]) + " << 8 | " + str(raw_temp[1]) + " = " + str(UT))
            print '='*40
            print
        
        if options.debug:
            print colors.OKGREEN + "OK" + colors.ENDC
        
    except Exception, ex:
        print colors.FAIL + "FAIL" + colors.ENDC
        print '='*40
        print str(ex)
        sys.exit(1)  
        
        
    if options.debug or options.e:
        print (colors.BOLD + "Reading uncompensated pressure value: " + colors.ENDC),
    try:
        #send command
        if options.e:
            print
            print("Sending command: "),
        BUS.write_i2c_block_data(options.address, 0xF4, [0x34+(options.oss<<6)])
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #wait
        if options.e:
            print("Sleep: "),
        if options.oss == 0:
            sleep(0.05)
        elif options.oss == 1:
            sleep(0.08)
        elif options.oss == 2:
            sleep(0.14)
        else:
            sleep(0.26)
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #read data
        if options.e:
            print("Reading data: "),
        raw_press = BUS.read_i2c_block_data(options.address, 0xF6, 3)
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #parse raw temperature
        if options.e:
            print("Converting raw data: "),
        global UP
        UP = (raw_press[0] << 16 | raw_press[1] << 8 | raw_press[0]) >> (8 - options.oss)
        if options.e:
            print colors.OKGREEN + "OK" + colors.ENDC
            
        #print temperature
        if options.e:
            print
            print("Temperature data:")
            print '='*40
            print("UP = (" + str(raw_press[0]) + " << 16 | " + str(raw_press[1]) + " << 8 | " + str(raw_press[2]) + ") >> (8 - " + str(options.oss) + ") = " + str(UP))
            print '='*40
            print
        
        if options.debug:
            print colors.OKGREEN + "OK" + colors.ENDC
        
    except Exception, ex:
        print colors.FAIL + "FAIL" + colors.ENDC
        print '='*40
        print str(ex)
        sys.exit(1)  
        
        
    return
        
    
def calculate_values():
    
    #calculate temeprature
    if options.debug or options.e:
        print (colors.BOLD + "Calculating temperature: " + colors.ENDC),
    X1 = (UT - AC6)*AC5/pow(2, 15)
    if options.e:
        print
        print '='*40
        print "X1: " + str(X1)
    X2 = MC*pow(2, 11)/(X1 + MD)
    if options.e:
        print "X2: " + str(X2)
    B5 = X1 + X2
    if options.e:
        print "B5: " + str(B5)
        print '='*40
    T = (B5 + 8)/pow(2, 4)
    if options.debug:
        print( colors.OKBLUE + colors.BOLD +  str(T*0.1) + "C" + colors.ENDC)
    else:
        print (colors.BOLD + "Temperature: " + colors.OKBLUE + str(T*0.1) + "C" + colors.ENDC)
        if options.e:
            print
        
        
    #calculate pressure
    if options.debug or options.e:
        
        print (colors.BOLD + "Calculating pressure: " + colors.ENDC),
    B6 = B5 - 4000
    if options.e:
        print
        print '='*40
        print "B6: " + str(B6)
    X1 = (B2*((B6*B6)/pow(2, 12)))/pow(2, 11)
    if options.e:
        print "X1: " + str(X1)
    X2 = AC2*B6/pow(2, 11)
    if options.e:
        print "X2: " + str(X2)
    X3 = X1+X2
    if options.e:
        print "X3: " + str(X3)
    B3 = (((AC1 * 4 + X3) << options.oss) + 2) / 4
    if options.e:
        print "B3: " + str(B3)
    X1 = (AC3*B6/pow(2, 13))
    if options.e:
        print "X1: " + str(X1)
    X2 = (B1 * (B6 * B6 / pow(2,12)))/pow(2, 16)
    if options.e:
        print "X2: " + str(X2)
    X3 = ((X1 + X2) + 2)/4
    if options.e:
        print "X3: " + str(X3)
    B4 = (AC4 * (X3 + 32768))/ pow(2, 15)
    if options.e:
        print "B4: " + str(B4)
    B7 = (UP - B3) * (50000 >> options.oss)
    if options.e:
        print "B7: " + str(B7)
    if B7 < 0x80000000:
        p = (B7 * 2) / B4
    else:
        p = (B7 / B4) * 2
    if options.e:
        print "p: " + str(p)
    X1 = (p/pow(2, 8)) * (p/pow(2, 8))
    if options.e:
        print "X1: " + str(X1)
    X1 = (X1 * 3038)/pow(2, 16)
    if options.e:
        print "X1: " + str(X1)
    X2 = (-7357*p)/pow(2, 16)
    if options.e:
        print "X2: " + str(X2)
    p = p + (X1 + X2 + 3791)/pow(2, 4)
    if options.e:
        print "p: " + str(p)
        print '='*40
    
    if options.debug:
        print( colors.OKBLUE + colors.BOLD +  str(p*0.01) + "hPa" + colors.ENDC)
    else: 
        print (colors.BOLD + "Pressure: " + colors.OKBLUE + str(p*0.01) + "hPa" + colors.ENDC)
        if options.e:
            print
        
    #calculate altitude
    if options.debug or options.e:        
        print (colors.BOLD + "Calculating pressure: " + colors.ENDC),
        
 
    if options.e:
        print
        print ("Converting altitude: "),
    altitude = 44330.0 * (1.0 - pow((p*1.0) / 101325, 0.1903))
    if options.e:
        print (colors.OKGREEN + "OK" + colors.ENDC)
    if options.debug:
        print( colors.OKBLUE + colors.BOLD +  str(altitude) + "m" + colors.ENDC)
    else: 
        print (colors.BOLD + "Altitude: " + colors.OKBLUE + str(altitude) + "m" + colors.ENDC)
        if options.e:
            print
    
    return
def main():
    print 
  #  print(sys.version)
    init()
    read_data()
    calculate_values()
    return 0

if __name__ == '__main__':
	main()

